
/**
 * @author isaac.fraile@i2cat.net
 */

/************************************************************************************
	TSService, llibreria per gestionar el servei TS del protocol DVB-CSS
	Aquest servei s'encarrega de distribuir els temps de reproducció i el temps de 
	rellotge per poder sincronitzar els continguts. 
	La comunicació es fa utilitzant websockets TCP
************************************************************************************/

var TSService = {

	SOCKET_LIST : [],
	CONTENT_ID : undefined,
	START_TIME : undefined,
	SPEED: 1,

	tsPromise : function(clock) {

	    return new Promise((resolve) => {

	    	var obj = {};
	    	
	    	obj.contentTime = clock.getTicks() - this.START_TIME;
	    	obj.wallClockTime = clock.getTicks();
	    	obj.timelineSpeedMultiplier = this.SPEED;

	        resolve( JSON.stringify( obj ) );
	    });
	},

	init : function(clock) {

		console.log("Init TS Service");
		this.START_TIME = clock.getTicks();
		this.SPEED = 1;
	},

	reset : function() {

		console.log("Reset TS Service");
		this.SOCKET_LIST = [];
		this.START_TIME = undefined;
		this.SPEED = 1.0;
		this.CONTENT_ID = undefined;
	},

	sendToAll : function(clock) {

		if ( this.SOCKET_LIST.length === 0 && this.CONTENT_ID ) this.reset();

		else 
		{
			this.tsPromise( clock ).then(( str ) => { 
				this.SOCKET_LIST.forEach( (e) => { e.send( str ) } );
			});
		}	
	},

	removeSocket : function(ws) {

		console.log('\x1b[31m->\x1b[0m Close TS connection');
		return new Promise((resolve) => {
			this.SOCKET_LIST.forEach( (e, index) => {
				if ( e === ws ) this.SOCKET_LIST.splice( index, 1 );
			});
			console.log( '\x1b[32m->\x1b[0m TS connections: ' + this.SOCKET_LIST.length );
			resolve( this.SOCKET_LIST.length );
	    });
	},

	removeAndCheck : function(ws) {

		return new Promise((resolve) => {
			this.removeSocket( ws ).then(( n ) => { 
				if ( n == 0 && this.CONTENT_ID ) this.reset();
			});
			resolve();
		});
	},

	addSocket : function(ws) {

		console.log('\x1b[33m->\x1b[0m Add new TS connection');
		this.SOCKET_LIST.push( ws );
	},

	setContent : function(data, clock) {

		return new Promise((resolve) => {
			if ( data.contentId && this.CONTENT_ID != data.contentId ) 
			{
				//if ( this.CONTENT_ID ) this.reset();
				this.CONTENT_ID = data.contentId;
				this.init( clock );

				// reset and set new content
				this.START_TIME = clock.getTicks();
				this.SPEED = 1.0;

				console.log('New content setted → ' + this.CONTENT_ID);
				resolve( this.CONTENT_ID );
			}
			else if ( data.action && data.actionTime )
			{
				this.SPEED = data.action == 'play' ? 1 : 0;
				this.START_TIME = clock.getTicks() - data.actionTime;
				this.sendToAll( clock );
			}
		});
	}
}

module.exports = TSService;