
/**
 * @author isaac.fraile@i2cat.net
 */

/************************************************************************************
    TCPServer, llibreria per gestionar les conexions del protocol DVB-CSS.

    Aquest servei s'encarrega de gestionar totes les conexion websocket TCP.

    Els serveis implementats son:
    * DVB-CSS-CII
    * DVB-CSS-TS
    * DVB-CSS-CS
    * DVB-CSS-WC (servei adicional, originariament corre sobre UDP)
************************************************************************************/

const WebSocket = require('ws');

var CII = require('./protocol/cii');
var WC = require('./protocol/wc');
var TS = require('./protocol/ts');

 
class TCPServer {
	constructor(clock) {

		const wss = new WebSocket.Server({ port: process.env.DEFAULT_WSPORT });
		const tsInterval = setInterval(function() { TS.sendToAll( clock ) }, 1000);
		const interval = setInterval(function ping() {
		  	wss.clients.forEach(function each(ws) {
		    	if (ws.isAlive === false) {
		    		console.log('Closse connection -> ws.terminate')
		    		return ws.terminate();
		    	}	 
		    	ws.isAlive = false;
		    	ws.ping(noop);
		  	});
		}, 30000);

		wss.on('listening', () => {
		    console.log('--> DVB-CSS TCP Server listening on ' + '\x1b[32m' + process.env.DEFAULT_ADDR + ':' + wss.options.port + '\x1b[0m');
		});
 
		wss.on('connection', function connection(ws, req) {
		
			ws.isAlive = true;
  			ws.on('pong', heartbeat);

  			checkTCPService( ws, req.url );

			ws.on('error', (error) => {
		    	console.log('\x1b[31m' + 'client→server connection socket error:' + '\x1b[0m', error);
		    });
		});

		function noop() {}
 
		function heartbeat() { this.isAlive = true }		
	
		function checkTCPService(ws, service)
		{
			console.log('New socket connection → Service: ' + service);
			console.log(ws._socket.remoteAddress)
			switch ( service ) {

				// DVB-CSS-CII Service
				case '/dvbcsscii':
					
					CII.addSocket( ws, TS.CONTENT_ID );
					ws.on('close', () => { CII.removeSocket( ws ) });

					break;

				// DVB-CSS-TS Service
				case '/dvbcssts':

					ws.on('message', function incoming(message) { TS.addSocket( ws ) });
					ws.on('close', () => { 
						TS.removeAndCheck( ws ).then(() => { if ( TS.CONTENT_ID == undefined ) CII.sendToAll() });
					});

					break;

				// DVB-CSS-WC Service
				case '/wc':

					ws.on('message', function incoming(message) {
						WC.WCService.wcPromise( parseFloat( message ), clock.getTicks(), clock ).then(( str ) => { ws.send(str) });
					});
					ws.on('close', () => { console.log('\x1b[31m->\x1b[0m Close WC connection') });

					break;

				// DVB-CSS-CS Service
				case '/cs':

					ws.on('message', function incoming(message) {
						TS.setContent( JSON.parse( message ), clock ).then(( str ) => { CII.sendToAll( str ) });
					});
					ws.on('close', () => { console.log('\x1b[31m->\x1b[0m Close CS connection') });

					break;

				// Experimental
				case '/log':

					ws.on('message', function incoming(message) {
						console.log( JSON.parse( message ) );
					});
					ws.on('close', () => { console.log('\x1b[31m->\x1b[0m Close LOG connection') });

					break;

				default: 
					console.log('Service unknown')
					break;
			}
		}
	}
}

module.exports = TCPServer;