
/**
 * @author isaac.fraile@i2cat.net
 */

/************************************************************************************
    Servidor per poder oferir sincronització

    Aquest servidor inicialitza el seu rellotge principal amb un temps NTP
    Inicialitza tots els serveis implementats del protocol DVB-CSS (servidor UDP i TCP)
    Defineix els ports d'entrada de cada servei
************************************************************************************/

"use strict"

const ntpsync = require( 'ntpsync' );
const Clock = require( './dvbcss/clock' );
const UDPServer = require( './dvbcss/udpserver' );
const TCPServer = require( './dvbcss/tcpserver' );

var ifs = require( 'os' ).networkInterfaces();

process.env.DEFAULT_WCPORT = 5000;
process.env.DEFAULT_WSPORT = 4649;
process.env.DEFAULT_PPM = 500;


// Get the local active IP
process.env.DEFAULT_ADDR = Object.keys( ifs )
    .map( x => ifs[x].filter( x => x.family === 'IPv4' && !x.internal )[0] )
    .filter( x => x )[0].address;

// Get NTP time and initialize the system
ntpsync.ntpLocalClockDeltaPromise().then((iNTPData) => {

    var _sysclock = new Clock( iNTPData );
    
    console.log(`(Local Time - NTP Time) Delta = ${iNTPData.minimalNTPLatencyDelta} ms`);
    console.log(`Minimal Ping Latency was ${iNTPData.minimalNTPLatency} ms`);
    console.log(`Total ${iNTPData.totalSampleCount} successful NTP Pings`);

    new UDPServer( _sysclock );
    new TCPServer( _sysclock );

}).catch((err) => {
    console.error(err);
}); 
